#include "3ds_objectloader.h"

C3DS_ObjectLoader::C3DS_ObjectLoader(void)
{
	m_3DModel.numOfMaterials = 0;
	m_3DModel.numOfObjects = 0;
}

void C3DS_ObjectLoader::Init(char *filename)
{
	// Load our .3DS file into our model structure
	m_Load3ds.Import3DS(&m_3DModel, filename);

	// Process textures
	COGLTexture texture;
	for(int i = 0; i < m_3DModel.numOfMaterials; i++)
	{
		// Check to see if there is a file name to load in this material
		if(strlen(m_3DModel.pMaterials[i].strFile) > 0)
		{
			texture.LoadFromFile(m_3DModel.pMaterials[i].strFile);
			m_Texture[i] = texture.GetID();
		}

		// Set the texture ID for this material
		m_3DModel.pMaterials[i].texureId = i;
	}

	m_ObjectDL = glGenLists(1);
	glNewList(m_ObjectDL, GL_COMPILE);
		for(int i = 0; i < m_3DModel.numOfObjects; i++)
		{
			// Make sure we have valid objects just in case. (size() is in the vector class)
			if(m_3DModel.pObject.size() <= 0) break;

			// Get the current object that we are displaying
			t3DObject *pObject = &m_3DModel.pObject[i];

			// Check to see if this object has a texture map, if so bind the texture to it.
			if(pObject->bHasTexture)
			{

				// Turn on texture mapping and turn off color
				glEnable(GL_TEXTURE_2D);

				// Reset the color to normal again
				glColor3ub(255, 255, 255);

				// Bind the texture map to the object by it's materialID
				glBindTexture(GL_TEXTURE_2D, m_Texture[pObject->materialID]);
			}
			else
			{

			// Turn off texture mapping and turn on color
			glDisable(GL_TEXTURE_2D);

			// Reset the color to normal again
			glColor3ub(255, 255, 255);
		}

		// This determines if we are in wireframe or normal mode
		glBegin(GL_TRIANGLES);					// Begin drawing with our selected mode (triangles or lines)

		// Go through all of the faces (polygons) of the object and draw them
		for(int j = 0; j < pObject->numOfFaces; j++)
		{
			// Go through each corner of the triangle and draw it.
			for(int whichVertex = 0; whichVertex < 3; whichVertex++)
			{
				// Get the index for each point of the face
				int index = pObject->pFaces[j].vertIndex[whichVertex];

				// Give OpenGL the normal for this vertex.
				glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);

				// If the object has a texture associated with it, give it a texture coordinate.
				if(pObject->bHasTexture) {

					// Make sure there was a UVW map applied to the object or else it won't have tex coords.
					if(pObject->pTexVerts) {
						glTexCoord2f(pObject->pTexVerts[ index ].x, pObject->pTexVerts[ index ].y);
					}
				} else {

					// Make sure there is a valid material/color assigned to this object.
					// You should always at least assign a material color to an object, 
					// but just in case we want to check the size of the material list.
					// if the size is at least one, and the material ID != -1,
					// then we have a valid material.
					if(m_3DModel.pMaterials.size() && pObject->materialID >= 0) 
					{
						// Get and set the color that the object is, since it must not have a texture
						BYTE *pColor = m_3DModel.pMaterials[pObject->materialID].color;

						// Assign the current color to this model
						glColor3ub(pColor[0], pColor[1], pColor[2]);
					}
				}

				// Pass in the current vertex of the object (Corner of current face)
				glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
			}
		}

		glEnd();								// End the drawing
	}
	glEndList();
}

C3DS_ObjectLoader::~C3DS_ObjectLoader(void)
{
	// delete display list
	glDeleteLists(m_ObjectDL, 1);

	// free the faces, normals, vertices, and texture coordinates.
	for(int i = 0; i < m_3DModel.numOfObjects; i++)
	{
		delete [] m_3DModel.pObject[i].pFaces;
		delete [] m_3DModel.pObject[i].pNormals;
		delete [] m_3DModel.pObject[i].pVerts;
		delete [] m_3DModel.pObject[i].pTexVerts;
	}
}

void C3DS_ObjectLoader::Draw(void)
{
	glCallList(m_ObjectDL);
}
