// 3D Vector Class by Arsham Skrenes (c)2005
// CPSC 314 - University of British Columbia

#include <cmath>
#include <iostream>

#ifndef C3DVECTOR_H
#define C3DVECTOR_H

class C3dVector
{
public:
	// Constructors and Destructors
	C3dVector(void);
	C3dVector(double x, double y, double z);
	C3dVector(double x, double y, double z, bool auto_normalize);
	C3dVector(const C3dVector &other);
	~C3dVector(void);

	// Set/Get components of vector
	void setXYZ(double x_value, double y_value, double z_value);

	// The following three functions will NOT auto normalize due to excessivly high
	// risk of incorrect answer if used incorrectly!
	void setX(double x_value);
	void setY(double y_value);
	void setZ(double z_value);
	
	double getX(void);
	double getY(void);
	double getZ(void);
	double getLength(void);

	// Normalization functions
	void normalize(void);
	void autoNormalize(bool set);

	// Operators
	void operator= (const C3dVector other);
	bool operator== ( const C3dVector other);
	bool operator!= ( const C3dVector other);
	void operator+= (const C3dVector other);
	void operator-= (const C3dVector other);
	C3dVector operator+ (const C3dVector other);
	C3dVector operator- (const C3dVector other);
	C3dVector operator* (const C3dVector other);
	C3dVector operator* (double scalar);		// cross product
	double dotProduct(const C3dVector other);	// dot product
	C3dVector component_wise_multiply(const C3dVector other);
	
	friend std::ostream& operator<< (std::ostream& pStr, const C3dVector& other);

private:
	double x, y, z;			// components of vector
	bool auto_normalize;	// if true, make sure vector is normalized (unless 0 vector)
};

#endif
