//
//
//	HillTerrain.inl
//
//
//	a class for a heightmap-based terrain generated using the hill algorithm.
//
//	change log:
//	--------
//	10/10/01	bob	first completion
//
//	----------------------------------------------------------------------------
//	copyright (c) 2001 bob nystrom                  [see license at end of file]

#include "3DVector.h"
#include <assert.h>
#include <stdlib.h>	// for NULL

//	GetSize
//	----------------------------------------------------------------------------
inline unsigned int CHillTerrain::GetSize( void )
{
	return( m_uSize );
}


//	GetHillMin
//	----------------------------------------------------------------------------
inline float
CHillTerrain::GetHillMin( void )
{
	return( m_fHillMin );
}


//	SetHillMin
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetHillMin( float fHillMin )
{
	// check the parameters
	assert( fHillMin >= 0.0 );
	assert( fHillMin < m_fHillMax );
	
	m_fHillMin = fHillMin;
}


//	GetHillMax
//	----------------------------------------------------------------------------
inline float
CHillTerrain::GetHillMax( void )
{
	return( m_fHillMax );
}


//	SetHillMax
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetHillMax( float fHillMax )
{
	// check the parameters
	assert( m_fHillMin < fHillMax );
	assert( fHillMax >= 0.0 );
	
	m_fHillMax = fHillMax;
}


//	GetMaxHillHeight
//	----------------------------------------------------------------------------
inline float
CHillTerrain::GetMaxHillHeight( void )
{
	return( m_fMaxHillHeight );
}


//	SetMaxHillHeight
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetMaxHillHeight( float fMaxHillHeight )
{	
	m_fMaxHillHeight = fMaxHillHeight;
	GenerateNormals();
}


//	SetHillSize
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetHillSize( float fHillMin, float fHillMax )
{
	// check the parameters
	assert( fHillMin >= 0.0 );
	assert( fHillMin < fHillMax );
	assert( fHillMax >= 0.0 );
	
	m_fHillMin = fHillMin;
	m_fHillMax = fHillMax;
}


//	GetNumHills
//	----------------------------------------------------------------------------
inline unsigned int
CHillTerrain::GetNumHills( void )
{
	return( m_uNumHills );
}


//	SetNumHills
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetNumHills( unsigned int uNumHills )
{
	m_uNumHills = uNumHills;
}


//	GetFlattening
//	----------------------------------------------------------------------------
inline unsigned int
CHillTerrain::GetFlattening( void )
{
	return( m_uFlattening );
}


//	SetFlattening
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetFlattening( unsigned int uFlattening )
{
	// check the parameters
	assert( uFlattening > 0 );
	
	m_uFlattening = uFlattening;
}


//	IsIsland
//	----------------------------------------------------------------------------
inline bool
CHillTerrain::IsIsland( void )
{
	return( m_bIsland );
}


//	SetIsland
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetIsland( bool bIsland )
{
	m_bIsland = bIsland;
}


//	GetSeed
//	----------------------------------------------------------------------------
inline unsigned int
CHillTerrain::GetSeed( void )
{
	return( m_uSeed );
}


//	SetSeed
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetSeed( unsigned int uSeed )
{
	m_uSeed = uSeed;
}


//	GetCell
//	----------------------------------------------------------------------------
inline float
CHillTerrain::GetCell( int x, int z )
{		
	return GetUnscaledCell(x, z)*m_fMaxHillHeight ;
}


//	GetCell Unscaled
//	----------------------------------------------------------------------------
inline float
CHillTerrain::GetUnscaledCell( int x, int z )
{
	// make sure we have a terrain
	assert( m_pMap != NULL );
	
	// check the parameters
	assert( ( x >= 0 ) && ( x < m_uSize ) );
	assert( ( z >= 0 ) && ( z < m_uSize ) );
		
	return( m_pMap[ x + ( z * m_uSize ) ] );
}


//	SetNormal
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetNormal( int x, int z, C3dVector value )
{
	// make sure we have a terrain and normals
	assert( m_pMap != NULL );
	assert( m_pNormals != NULL );
	
	// check the parameters
	assert( ( x >= 0 ) && ( x < m_uSize ) );
	assert( ( z >= 0 ) && ( z < m_uSize ) );
	
	// set the cell (and make sure it's normalized)
	value.normalize();
	m_pNormals[ x + ( z * m_uSize ) ] = value;
}

//	GetNormal
//	----------------------------------------------------------------------------
inline C3dVector
CHillTerrain::GetNormal( int x, int z )
{
	// make sure we have a vertices
	assert( m_pNormals != NULL );
	
	// check the parameters
	assert( ( x >= 0 ) && ( x < m_uSize ) );
	assert( ( z >= 0 ) && ( z < m_uSize ) );
		
	return( m_pNormals[ x + ( z * m_uSize ) ] );
}


//	SetCell
//	----------------------------------------------------------------------------
inline void
CHillTerrain::SetCell( int x, int z, float value )
{
	// make sure we have a terrain
	assert( m_pMap != NULL );
	
	// check the parameters
	assert( ( x >= 0 ) && ( x < m_uSize ) );
	assert( ( z >= 0 ) && ( z < m_uSize ) );
	
	// set the cell
	m_pMap[ x + ( z * m_uSize ) ] = value;
}


//	OffsetCell
//	----------------------------------------------------------------------------
inline void
CHillTerrain::OffsetCell( int x, int z, float value )
{
	// make sure we have a terrain
	assert( m_pMap != NULL );
	
	// check the parameters
	assert( ( x >= 0 ) && ( x < m_uSize ) );
	assert( ( z >= 0 ) && ( z < m_uSize ) );

	// offset the cell
	m_pMap[ x + ( z * m_uSize ) ] += value;
}

//	license agreement:
//
//	----------------------------------------------------------------------------
//	copyright (c) 2001 bob nystrom
//
//	permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "software"), to
//	deal in the software without restriction, including without limitation the
//	rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
//	sell copies of the software, and to permit persons to whom the software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	the software is provided "as is", without warranty of any kind, express or
//	implied, including but not limited to the warranties of merchantability,
//	fitness for a particular purpose and noninfringement. in no event shall the
//	authors or copyright holders be liable for any claim, damages or other
//	liability, whether in an action of contract, tort or otherwise, arising
//	from, out of or in connection with the software or the use or other dealings
//	in the software.
//	----------------------------------------------------------------------------
//	see www.opensource.org for more info.