#include "MissilePicker.h"

void MissilePicker::Initialize(void)
{
	// ______________________________________________
	// Create a very efficient cube as a display list
	m_ObjectDL = glGenLists(1);
	glNewList(m_ObjectDL,GL_COMPILE);
		glBegin(GL_TRIANGLE_STRIP);
			glVertex3f(	-0.5, -0.5, -0.5 );
			glVertex3f(	-0.5, -0.5, +0.5 );
			glVertex3f(	+0.5, -0.5, -0.5 );
			glVertex3f(	+0.5, -0.5, +0.5 );
			glVertex3f(	+0.5, +0.5, -0.5 );
			glVertex3f(	+0.5, +0.5, +0.5 );
			glVertex3f(	-0.5, +0.5, -0.5 );
			glVertex3f(	-0.5, +0.5, +0.5 );
		glEnd();
		glBegin(GL_TRIANGLE_STRIP);
			glVertex3f(	+0.5, +0.5, +0.5 );
			glVertex3f(	+0.5, -0.5, +0.5 );
			glVertex3f(	-0.5, +0.5, +0.5 );
			glVertex3f(	-0.5, -0.5, +0.5 );
			glVertex3f(	-0.5, +0.5, -0.5 );
			glVertex3f(	-0.5, -0.5, -0.5 );
			glVertex3f(	+0.5, +0.5, -0.5 );
			glVertex3f(	+0.5, -0.5, -0.5 );
		glEnd();
	glEndList();
}

void MissilePicker::Render(const int cursorX, const int cursorY)
{
	StartPicking(cursorX, cursorY);

	// render boundary boxes for blocks
	int i;
	for(i = 0; i < BLOCK_NUM; i++)
	{
		if(tanks[i] != NULL)
		{
			glPushMatrix();
				glPushName(i);
				glTranslatef( blocks[i]->pos.x, blocks[i]->pos.y, blocks[i]->pos.z );
				glScalef( blocks[i]->length, blocks[i]->height, blocks[i]->width );
				glCallList(m_ObjectDL);
				glPopName();
			glPopMatrix();
		}
	}

	// render boundary boxes for tanks
	for(; i < ENEMY_NUM+BLOCK_NUM; i++)
	{
		if(tanks[i] != NULL)
		{
			glPushMatrix();
				glPushName(i);
				glTranslatef( tanks[i]->pos.x, tanks[i]->pos.y, tanks[i]->pos.z );
				glScalef( 1.5*(tanks[i]->radius), 1.5*tanks[i]->radius, 1.5*tanks[i]->radius );
				glCallList(m_ObjectDL);
				glPopName();
			glPopMatrix();
		}
	}
	StopPicking();
}

bool MissilePicker::GetPickedObject(float &objectX, float &objectY, float &objectZ)
{
	if (m_Hits == 0) return false;
		
	int obj_num = ProcessHits();

	if (obj_num < BLOCK_NUM)
	{
		objectX = blocks[obj_num]->pos.x;
		objectY = blocks[obj_num]->pos.y;
		objectZ = blocks[obj_num]->pos.z;
	}
	else
	{
		objectX = tanks[obj_num-BLOCK_NUM]->pos.x;
		objectY = tanks[obj_num-BLOCK_NUM]->pos.y;
		objectZ = tanks[obj_num-BLOCK_NUM]->pos.z;
	}

	m_Hits = 0;
	return true;
}

void MissilePicker::StartPicking(const int &cursorX, const int &cursorY)
{
	GLint viewport[4];
	float ratio;

	glSelectBuffer(BUFFERSIZE,m_Buffer);

	glGetIntegerv(GL_VIEWPORT,viewport);

	glRenderMode(GL_SELECT);
	glInitNames();

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	gluPickMatrix(cursorX,viewport[3]-cursorY,1,1,viewport);
	ratio = (viewport[2]+0.0) / viewport[3];
	gluPerspective(45,ratio,0.1,10000);
	glMatrixMode(GL_MODELVIEW);
}

void MissilePicker::StopPicking(void)
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glFlush();

	m_Hits = glRenderMode(GL_RENDER);
}

GLuint MissilePicker::ProcessHits(void)
{
	GLuint names, *ptr, minZ,*ptrName;

	ptr = (GLuint *) m_Buffer;
	minZ = 0xffffffff;
	for (int i = 0; i < m_Hits; i++)
	{
		names = *ptr;
		ptr++;

		// if this object is closer, select this one instead
		if (*ptr < minZ)
		{
			minZ = *ptr;
			ptrName = ptr+2;
		}

		ptr += names+2;
	}

	// return the ID of the closest object picked
	return *ptrName;
}
