#pragma once

#include <windows.h>
#include <conio.h>
#include "HillTerrain.h"
#include "Textures.h"

#define GL_CLAMP_TO_EDGE	0x812F	// This is for our skybox textures (OpenGL 1.2)
#define SECTOR				127		// the width/height of a sector (NOT # VERTICES!!)

class CTerrainSector
{
public:
	CTerrainSector(	unsigned short	uSize,
					float			fHillMin,
					float			fHillMax,
					float			fMaxHillHeight,
					unsigned short	uNumHills,
					unsigned short	uFlattening,
					bool			bIsland,
					unsigned int	uSeed,
					GLuint			uPlainDL,
					unsigned int	uTextureID );
	~CTerrainSector(void);

	// create/destroy/draw sector
	void create(bool &isThreading);	// isThreading must be false to create a new thread
	void destroy(bool &isThreading);
	void draw(void);

	// Returns y value of plane (collision point) based on x and z value
	// PRE: x and z cannot be >= m_Size-1
	double CollisionPoint(float x, float z);

private:
	// This is the function I call as a new thread
	static void ThreadedGenerator(CTerrainSector *pointer);
	HANDLE			m_Thread;		// handle for thread
	bool			m_HandleMe;		// does the thread handle need to be closed?
	bool			m_IsCreating;	// is a thread running create_helper?
	bool			m_IsCreated;	// has sector been made?
	
	GLuint			m_PlainDL;		// display list for a plain
	GLuint			m_TerrainDL;	// display list for terrain
	CHillTerrain*	m_Sector;
	unsigned int	m_TextureID;

	unsigned short	m_Size;
	float			m_HillMin;
	float			m_HillMax;
	float			m_MaxHillHeight;
	unsigned short	m_NumHills;
	unsigned short	m_Flattening;
	bool			m_Island;
	unsigned int	m_Seed;
};

class CTerrainMap
{
public:
	// BEWARE: this constructor will initialize and enable fog for the terrain!
	// visibility_detail ranges from 0 - 20; 0 is least and 20 is maximum
	// BEWARE: most computers cannot handle 20 - it also consumes well over 100mb of RAM!!
	// provide same random_seed for same terrain, or different for different terrain.
	CTerrainMap(unsigned short visibility_detail, unsigned int random_seed );
	~CTerrainMap(void);
	
	// draw map based on aprox camera location in (X,Z) coordinates
	// (y coordinate doesn't matter)
	void draw(int camera_X, int camera_Z);
	
	// returns y value of plain (collision point) based on x and z value
	// returns -1 if out of plain
	double CollisionPoint(float x, float z);
	
	// used with CollisionPoint() for a 4 point vehicle
	// x1=front; x2=back; x3=left-side; x4=right-side;
	// returns pitch & roll angles of vehicle
	void VehicleCollision(float x1, float z1, float x2, float z2, float x3, float z3, float x4, float z4, float& pitch, float& roll);

	// returns true if generation thread currently running
	bool PendingThread(void);

private:
	void CreateSkyBox(float x, float y, float z, float width, float height, float length);
	CTerrainSector*	m_grid[20][20];											// holds over 5sq. km of terrain
	GLuint			m_dList;												// display lists (401 in total)
	bool			m_isThreading;											// is there a generation thread executing?
	int				m_HighBound1, m_HighBound2, m_LowBound1, m_LowBound2;	// visibility
	COGLTexture		m_Tex1, m_Tex2, m_Tex3, m_Tex4, m_Tex5, m_Desert_Texture;
};
